@@ -344,8 +344,8 @@ void Fork_Initializer(void)
 	 * When implemented...  Although logout_all should clear this up.
 	 */
 
-	bt_destroy(&priv_token_obj_btree, object_free);
-	bt_destroy(&publ_token_obj_btree, object_free);
+	bt_destroy(&priv_token_obj_btree, call_free);
+	bt_destroy(&publ_token_obj_btree, call_free);
 
 	/* Need to do something to prevent the shared memory from
 	 * having the objects loaded again.... The most likely place
@@ -605,7 +605,7 @@ CK_RV SC_GetMechanismList(CK_SLOT_ID sid, CK_MECHANISM_TYPE_PTR pMechList,
 	}
 out:
 	TRACE_INFO("C_GetMechanismList:  rc = 0x%08lx, # mechanisms: %lu\n",
-		    rc, *count);
+		    rc, (count ? *count : 0));
 	return rc;
 }
 
@@ -1082,12 +1082,15 @@ done:
 		rc = session_mgr_login_all(userType);
 		if (rc != CKR_OK)
 			TRACE_DEVEL("session_mgr_login_all failed.\n");
-		else
-			rc = icsf_get_handles(sess->session_info.slotID);
+		else {
+			if (sess)
+				rc = icsf_get_handles(sess->session_info.slotID);
+		}
 	}
 
 	TRACE_INFO("C_Login: rc = 0x%08lx\n", rc);
-	save_token_data(sess->session_info.slotID);
+	if (sess)
+		save_token_data(sess->session_info.slotID);
 	MY_UnlockMutex(&login_mutex);
 	return rc;
 }
@@ -1256,13 +1259,44 @@ done:
 CK_RV SC_GetObjectSize(ST_SESSION_HANDLE *sSession, CK_OBJECT_HANDLE hObject,
 		       CK_ULONG_PTR pulSize)
 {
+	SESSION *sess = NULL;
+	CK_RV rc = CKR_OK;
+	/**
+	ock does not do object management for icsf token. To get the
+	object size call CSFPGAV and extract the attr_length returned.
+	icsf_get_attribute does not pass the user provided template
+	attributes to remote icsf, instead gets all the attributes from
+	remote icsf and returns only the user requested attributes.
+	icsf_get_object_size tries to do the same and extracts only the
+	attribute_list_length from the result. Setting attribute list to
+	NULL here and providing a dummy count value.
+	 **/
+	CK_ATTRIBUTE_PTR pTemplate = NULL;
+	CK_ULONG ulCount = 1;
+
 	if (initialized == FALSE) {
 		TRACE_ERROR("%s\n", ock_err(ERR_CRYPTOKI_NOT_INITIALIZED));
 		return CKR_CRYPTOKI_NOT_INITIALIZED;
 	}
 
-	TRACE_ERROR("%s\n", ock_err(ERR_FUNCTION_NOT_SUPPORTED));
-	return CKR_FUNCTION_NOT_SUPPORTED;
+	sess = session_mgr_find(sSession->sessionh);
+	if (!sess) {
+		TRACE_ERROR("%s\n", ock_err(ERR_SESSION_HANDLE_INVALID));
+		rc = CKR_SESSION_HANDLE_INVALID;
+		goto done;
+	}
+
+	rc = icsftok_get_attribute_value(sess, hObject, pTemplate,
+					ulCount, pulSize);
+	if (rc != CKR_OK)
+		TRACE_DEVEL("icsftok_get_attribute_value() failed.\n");
+
+
+done:
+	TRACE_INFO("C_GetObjectSize: rc = 0x%08lx, handle = %lu\n",
+		    rc, hObject);
+
+	return rc;
 }
 
 
@@ -1286,7 +1320,8 @@ CK_RV SC_GetAttributeValue(ST_SESSION_HANDLE *sSession,
 		goto done;
 	}
 
-	rc = icsftok_get_attribute_value(sess, hObject, pTemplate, ulCount);
+	rc = icsftok_get_attribute_value(sess, hObject, pTemplate,
+					ulCount, NULL);
 	if (rc != CKR_OK)
 		TRACE_DEVEL("icsftok_get_attribute_value() failed.\n");
 
@@ -1559,7 +1594,7 @@ CK_RV SC_EncryptInit(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism,
 done:
 	TRACE_INFO("C_EncryptInit: rc = 0x%08lx, sess = %ld, mech = 0x%lx\n",
 		   rc, (sess == NULL) ? -1 : (CK_LONG)sess->handle,
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 
 	return rc;
 }
@@ -1765,7 +1800,7 @@ CK_RV SC_DecryptInit(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism,
 done:
 	TRACE_INFO("C_DecryptInit: rc = 0x%08lx, sess = %ld, mech = 0x%lx\n",
 		   rc, (sess == NULL) ? -1 : (CK_LONG)sess->handle,
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 
 	return rc;
 }
@@ -1917,7 +1952,7 @@ done:
 
 	TRACE_INFO("C_DecryptFinal:  rc = 0x%08lx, sess = %ld, amount = %lu\n",
 		   rc, (sess == NULL) ? -1 : (CK_LONG)sess->handle,
-		   *pulLastPartLen);
+		   (pulLastPartLen ? *pulLastPartLen : -1));
 
 	return rc;
 }
@@ -1969,7 +2004,7 @@ CK_RV SC_DigestInit(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism)
 done:
 	TRACE_INFO("C_DigestInit: rc = 0x%08lx, sess = %ld, mech = %lx\n",
 		   rc, (sess == NULL)?-1:(CK_LONG)sess->handle,
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 
 	return rc;
 }
@@ -2205,7 +2240,7 @@ CK_RV SC_SignInit(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism,
 done:
 	TRACE_INFO("C_SignInit: rc = %08lx, sess = %ld, mech = %lx\n",
 		   rc, (sess == NULL)?-1:(CK_LONG)sess->handle,
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 
 	return rc;
 }
@@ -2426,7 +2461,7 @@ CK_RV SC_VerifyInit(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism,
 done:
 	TRACE_INFO("C_VerifyInit: rc = %08lx, sess = %ld, mech = %lx\n",
 		   rc, (sess == NULL)?-1:(CK_LONG)sess->handle,
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 
 	return rc;
 }
@@ -2698,7 +2733,7 @@ CK_RV SC_GenerateKey(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism,
 done:
 	TRACE_INFO("C_GenerateKey: rc = %08lx, sess = %ld, mech = %lu\n", rc,
 		    (sess == NULL) ? -1 : (CK_LONG) sess->handle,
-		    pMechanism->mechanism);
+		    (pMechanism ? pMechanism->mechanism : -1));
 
 #ifdef DEBUG
 	int i;
@@ -2774,7 +2809,7 @@ CK_RV SC_GenerateKeyPair(ST_SESSION_HANDLE *sSession,
 done:
 	TRACE_INFO("C_GenerateKeyPair: rc = %08lx, sess = %ld, mech = %lx\n",
 		   rc, (sess == NULL) ? -1 : ((CK_LONG) sess->handle),
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 
 #ifdef DEBUG
 	int i;
@@ -2912,7 +2947,7 @@ done:
 	TRACE_INFO("C_UnwrapKey: rc = %08lx, sess = %ld, decrypting key = %lu,"
 		   "unwrapped key = %lu\n", rc,
 		   (sess == NULL) ? -1 : (CK_LONG) sess->handle,
-		   hUnwrappingKey, *phKey);
+		   hUnwrappingKey, (phKey ? *phKey : -1));
 
 #ifdef DEBUG
 	int i;
@@ -2979,7 +3014,7 @@ CK_RV SC_DeriveKey(ST_SESSION_HANDLE *sSession, CK_MECHANISM_PTR pMechanism,
 done:
 	TRACE_INFO("C_DeriveKey: rc = %08lx, sess = %ld, mech = %lu\n",
 		   rc, (sess == NULL)?-1:(CK_LONG)sess->handle,
-		   pMechanism->mechanism);
+		   (pMechanism ? pMechanism->mechanism : -1));
 #ifdef DEBUG
 	int i;
 	CK_ATTRIBUTE *attr = NULL;
